#include <windows.h>
#include <shlobj.h>
#include "resource.h"
#include "common.h"
#include "rstring.h"
#include "dirrefer.h"

int WINAPI BrowseFolderCallback(HWND hwnd, UINT uMsg, LPARAM lParam, LPARAM lpData)
{
	switch(uMsg)
	{
	case BFFM_INITIALIZED:
		SendMessage(hwnd, BFFM_SETSELECTION, TRUE/*=Path, not IDL*/, lpData);
		break;
	}

	return 0;
}

BOOL BrowseFolder(HWND hWnd, LPTSTR path, LPCTSTR initPath)
{
	BROWSEINFO bi = {0};
	ITEMIDLIST* idl;

	bi.hwndOwner = hWnd;
	bi.ulFlags = BIF_RETURNONLYFSDIRS;
	bi.pidlRoot = (LPCITEMIDLIST)CSIDL_DESKTOP;
	bi.lpszTitle = String(IDS_SELECTDIR);
	bi.lpfn = BrowseFolderCallback;
	bi.lParam = (LPARAM)initPath;

	if(idl = SHBrowseForFolder(&bi))
	{
		BOOL ret;

		ret = SHGetPathFromIDList(idl, path);

		CoTaskMemFree(idl);

		if(ret)
			return TRUE;
	}

	return FALSE;
}

LRESULT CALLBACK DirboxSubProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
	case WM_APP:
		SetWindowLong(hWnd, GWL_USERDATA, GetWindowLong(hWnd, GWL_WNDPROC));
		SetWindowLong(hWnd, GWL_WNDPROC, (LONG)DirboxSubProc);
		return 0;

	case WM_DROPFILES:
		//CallWindowProc((WNDPROC)GetWindowLong(GetParent(hWnd), GWL_WNDPROC), hWnd, msg, wParam, lParam);
		SendMessage(GetParent(hWnd), msg, wParam, lParam);
		return 0;
	}

	return CallWindowProc((WNDPROC)GetWindowLong(hWnd, GWL_USERDATA), hWnd, msg, wParam, lParam);
}

BOOL CALLBACK DirreferDlgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch(msg)
	{
	case WM_INITDIALOG:
		{
			struct tagDIRREFERDLG* drd = (void*)lParam;

			if(drd->desc)
				SetDlgItemText(hWnd, IDC_DESCRIPTION, (LPCTSTR)drd->desc);

			if(drd->initDir)
				SetDlgItemText(hWnd, IDC_DIR, (LPCTSTR)drd->initDir);

			if(drd->checkRecursive)
				SendMessage(GetDlgItem(hWnd, IDC_RECURSIVE), BM_SETCHECK, TRUE, 0);
			else
				ShowWindow(GetDlgItem(hWnd, IDC_RECURSIVE), SW_HIDE);

			DirboxSubProc(GetDlgItem(hWnd, IDC_DIR), WM_APP, 0, 0);
			SetWindowLong(GetDlgItem(hWnd, IDC_DIR), GWL_EXSTYLE, GetWindowLong(GetDlgItem(hWnd, IDC_DIR), GWL_EXSTYLE) | WS_EX_ACCEPTFILES);
		}

		return TRUE;
	case WM_CLOSE:
		SendMessage(hWnd, WM_COMMAND, IDCANCEL, 0);
		break;
	case WM_DROPFILES:
		{
			TCHAR fn[MAX_PATH];

			DragQueryFile((HDROP)wParam, 0, fn, sizeof(fn));
			DragFinish((HDROP)wParam);

			SetDlgItemText(hWnd, IDC_DIR, fn);
		}
		break;
	case WM_COMMAND:
		switch(LOWORD(wParam))
		{
		case IDOK:
			{
				LPTSTR path;
				int len;
				int recurse = 0;

				len = (SendMessage(GetDlgItem(hWnd, IDC_DIR), WM_GETTEXTLENGTH, 0, 0) + 1) * sizeof(TCHAR);
				if(len - 1 == 0)
				{
					MessageBox(hWnd, String(IDS_NULLDIRECTORY), String(IDS_ERROR), MB_ICONERROR | MB_OK);
					break;
				}
				path = halloc(len);
				GetDlgItemText(hWnd, IDC_DIR, path, len);

				if(IsWindowVisible(GetDlgItem(hWnd, IDC_RECURSIVE)) && SendMessage(GetDlgItem(hWnd, IDC_RECURSIVE), BM_GETCHECK, 0, 0) == BST_CHECKED)
					recurse = 1;

				EndDialog(hWnd, (int)path | (recurse << 31));
			}
			break;
		case IDCANCEL:
			EndDialog(hWnd, (int)NULL);
			break;
		case IDC_REFER:
			{
				TCHAR path[MAX_PATH];

				GetDlgItemText(hWnd, IDC_DIR, path, sizeof(path));

				if(BrowseFolder(hWnd, path, path))
				{
					int len;

					SetDlgItemText(hWnd, IDC_DIR, path);
					len = SendMessage(GetDlgItem(hWnd, IDC_DIR), WM_GETTEXTLENGTH, 0, 0);
					SendMessage(GetDlgItem(hWnd, IDC_DIR), EM_SETSEL, len, len);
					SetFocus(GetDlgItem(hWnd, IDC_DIR));
				}
			}
			break;
		}

		break;
	}

	return FALSE;
}

LPTSTR openDirreferDlg(HWND hParentWnd, LPCTSTR desc, BOOL checkRecursive, LPCTSTR initDir)
{
	struct tagDIRREFERDLG drd;

	drd.desc = desc;
	drd.checkRecursive = checkRecursive;
	drd.initDir = initDir;

	return (LPTSTR)DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_REFERDIRECTORY), hParentWnd, DirreferDlgProc, (LPARAM)&drd);
}
